X-Git-Url: https://git.zerfleddert.de/cgi-bin/gitweb.cgi/proxmark3-svn/blobdiff_plain/fd1d30cb76a384adc8176df1f0f35658c091d96c..75e4ee2f465d4e86350e78de69a5612ad21eba3b:/client/cmddata.c?ds=inline

diff --git a/client/cmddata.c b/client/cmddata.c
index fd08b6dd..f3644304 100644
--- a/client/cmddata.c
+++ b/client/cmddata.c
@@ -23,10 +23,11 @@
 #include "lfdemod.h"
 #include "usb_cmd.h"
 #include "crc.h"
+#include "crc16.h"
 
 uint8_t DemodBuffer[MAX_DEMOD_BUF_LEN];
-uint8_t g_debugMode;
-size_t DemodBufferLen;
+uint8_t g_debugMode=0;
+size_t DemodBufferLen=0;
 static int CmdHelp(const char *Cmd);
 
 //set the demod buffer with given array of binary (one bit per byte)
@@ -40,7 +41,7 @@ void setDemodBuf(uint8_t *buff, size_t size, size_t startIdx)
 		size = MAX_DEMOD_BUF_LEN;
 
 	size_t i = 0;
-for (; i < size; i++){
+	for (; i < size; i++){
 		DemodBuffer[i]=buff[startIdx++];
 	}
 	DemodBufferLen=size;
@@ -343,8 +344,8 @@ int ASKDemod(const char *Cmd, bool verbose, bool emSearch, uint8_t askType)
 	setDemodBuf(BitStream,BitLen,0);
 	if (verbose || g_debugMode){
 		if (errCnt>0) PrintAndLog("# Errors during Demoding (shown as 7 in bit stream): %d",errCnt);
-		if (askType) PrintAndLog("ASK/Manchester decoded bitstream:");
-		else PrintAndLog("ASK/Raw decoded bitstream:");
+		if (askType) PrintAndLog("ASK/Manchester - Clock: %d - Decoded bitstream:",clk);
+		else PrintAndLog("ASK/Raw - Clock: %d - Decoded bitstream:",clk);
 		// Now output the bitstream to the scrollback by line of 16 bits
 		printDemodBuff();
 		
@@ -391,7 +392,7 @@ int Cmdmandecoderaw(const char *Cmd)
 	int errCnt=0;
 	size_t size=0;
 	int invert=0;
-	size_t maxErr = 20;
+	int maxErr = 20;
 	char cmdp = param_getchar(Cmd, 0);
 	if (strlen(Cmd) > 5 || cmdp == 'h' || cmdp == 'H') {
 		PrintAndLog("Usage:  data manrawdecode [invert] [maxErr]");
@@ -497,22 +498,20 @@ int CmdBiphaseDecodeRaw(const char *Cmd)
 int ASKbiphaseDemod(const char *Cmd, bool verbose)
 {
 	//ask raw demod GraphBuffer first
-	int offset=0, clk=0, invert=0, maxErr=0, ans=0;
-	ans = sscanf(Cmd, "%i %i %i %i", &offset, &clk, &invert, &maxErr);
-	if (ans>0)
-		ans = ASKDemod(Cmd+2, FALSE, FALSE, 0);
-	else
-		ans = ASKDemod(Cmd, FALSE, FALSE, 0);
-	if (!ans) {
-		if (g_debugMode || verbose) PrintAndLog("Error AskDemod: %d", ans);
-		return 0;
-	}
-
-	//attempt to Biphase decode DemodBuffer
-	size_t size = DemodBufferLen;
+	int offset=0, clk=0, invert=0, maxErr=0;
+	sscanf(Cmd, "%i %i %i %i", &offset, &clk, &invert, &maxErr);
+	
 	uint8_t BitStream[MAX_DEMOD_BUF_LEN];
-	memcpy(BitStream, DemodBuffer, DemodBufferLen); 
-	int errCnt = BiphaseRawDecode(BitStream, &size, offset, 0);
+	size_t size = getFromGraphBuf(BitStream);	  
+	//invert here inverts the ask raw demoded bits which has no effect on the demod, but we need the pointer
+	int errCnt = askdemod(BitStream, &size, &clk, &invert, maxErr, 0, 0);  
+	if ( errCnt < 0 || errCnt > maxErr ) {   
+		if (g_debugMode) PrintAndLog("DEBUG: no data or error found %d, clock: %d", errCnt, clk);  
+			return 0;  
+	} 
+
+	//attempt to Biphase decode BitStream
+	errCnt = BiphaseRawDecode(BitStream, &size, offset, invert);
 	if (errCnt < 0){
 		if (g_debugMode || verbose) PrintAndLog("Error BiphaseRawDecode: %d", errCnt);
 		return 0;
@@ -524,7 +523,7 @@ int ASKbiphaseDemod(const char *Cmd, bool verbose)
 	//success set DemodBuffer and return
 	setDemodBuf(BitStream, size, 0);
 	if (g_debugMode || verbose){
-		PrintAndLog("Biphase Decoded using offset: %d - # errors:%d - data:",offset,errCnt);
+		PrintAndLog("Biphase Decoded using offset: %d - clock: %d - # errors:%d - data:",offset,clk,errCnt);
 		printDemodBuff();
 	}
 	return 1;
@@ -1457,13 +1456,24 @@ int CmdFSKdemodPyramid(const char *Cmd)
 	return 1;
 }
 
 // ISO11784/85 demod  (aka animal tag)  BIPHASE, inverted, rf/32,  with preamble of 00000000001 (128bits)
 // 8 databits + 1 parity (1)
 // CIITT 16 chksum
 // NATIONAL CODE, ICAR database
-// COUNTRY CODE (ISO3166) 
+// COUNTRY CODE (ISO3166) or http://cms.abvma.ca/uploads/ManufacturersISOsandCountryCodes.pdf
 // FLAG (animal/non-animal)
-int CmdIso11784demodBI(const char *Cmd){
+/*
+38 IDbits   
+10 country code 
+1 extra app bit
+14 reserved bits
+1 animal bit
+16 ccitt CRC chksum over 64bit ID CODE.
+24 appli bits.
+
+-- sample: 985121004515220  [ 37FF65B88EF94 ]
+*/
+int CmdFDXBdemodBI(const char *Cmd){
 
 	int invert = 1;
 	int clk = 32;		
@@ -1474,7 +1484,7 @@ int CmdIso11784demodBI(const char *Cmd){
 	
 	errCnt = askdemod(BitStream, &size, &clk, &invert, maxErr, 0, 0);
 	if ( errCnt < 0 || errCnt > maxErr ) { 
-		if (g_debugMode) PrintAndLog("DEBUG: no data or error found %d, clock: 32", errCnt);
+		if (g_debugMode) PrintAndLog("DEBUG: no data or error found %d, clock: %d", errCnt, clk);
 		return 0;
 	}
 
@@ -1483,30 +1493,30 @@ int CmdIso11784demodBI(const char *Cmd){
 		if (g_debugMode) PrintAndLog("Error BiphaseRawDecode: %d", errCnt);
 		return 0;
 	} 
-
-	int preambleIndex = ISO11784demodBI(BitStream, &size);
+	
+	int preambleIndex = FDXBdemodBI(BitStream, &size);
 	if (preambleIndex < 0){
-		if (g_debugMode) PrintAndLog("Error ISO11784Demod , no startmarker found :: %d",preambleIndex);
+		if (g_debugMode) PrintAndLog("Error FDXBDemod , no startmarker found :: %d",preambleIndex);
 		return 0;
 	}
-
+	
 	setDemodBuf(BitStream, 128, preambleIndex);
-	//printDemodBuff();
 
+	// remove marker bits (1's every 9th digit after preamble) (pType = 2)
 	size = removeParity(BitStream, preambleIndex + 11, 9, 2, 117);
-	if ( size <= 103 ) {
+	if ( size != 104 ) {
 		if (g_debugMode) PrintAndLog("Error removeParity:: %d", size);
 		return 0;
 	}
-	//char *bin = sprint_bin_break(BitStream,size,16);
-	//PrintAndLog("DEBUG BinStream:\n%s",bin);
-
-	PrintAndLog("startmarker %d;   Size %d", preambleIndex, size);
+	if (g_debugMode) {
+		char *bin = sprint_bin_break(BitStream,size,16);
+		PrintAndLog("DEBUG BinStream:\n%s",bin);
+	}
+	PrintAndLog("\nFDX-B / ISO 11784/5 Animal Tag ID Found:");
+	if (g_debugMode) PrintAndLog("Start marker %d;   Size %d", preambleIndex, size);
 
-	//return 1;
 	//got a good demod
-	uint32_t NationalCodeA = bytebits_to_byteLSBF(BitStream,32);
-	uint32_t NationalCodeB = bytebits_to_byteLSBF(BitStream+32,6);
+	uint64_t NationalCode = ((uint64_t)(bytebits_to_byteLSBF(BitStream+32,6)) << 32) | bytebits_to_byteLSBF(BitStream,32);
 	uint32_t countryCode = bytebits_to_byteLSBF(BitStream+38,10);
 	uint8_t dataBlockBit = BitStream[48];
 	uint32_t reservedCode = bytebits_to_byteLSBF(BitStream+49,14);
@@ -1514,13 +1524,21 @@ int CmdIso11784demodBI(const char *Cmd){
 	uint32_t crc16 = bytebits_to_byteLSBF(BitStream+64,16);
 	uint32_t extended = bytebits_to_byteLSBF(BitStream+80,24);
 
-	PrintAndLog("NationalCode: %X%08X",NationalCodeB,NationalCodeA);
-	PrintAndLog("CountryCode:  %d",countryCode);
-	PrintAndLog("dataBlockBit: %d",dataBlockBit);
-	PrintAndLog("reservedCode: %X",reservedCode);
-	PrintAndLog("animalBit:    %d", animalBit);
-	PrintAndLog("CRC:          %02X", crc16);
-	PrintAndLog("Extended:     %x", extended);
+	uint64_t rawid = ((uint64_t)bytebits_to_byte(BitStream,32)<<32) | bytebits_to_byte(BitStream+32,32);
+	uint8_t raw[8];
+	num_to_bytes(rawid, 8, raw);
+
+	if (g_debugMode) PrintAndLog("Raw ID Hex: %s", sprint_hex(raw,8));
+
+	uint16_t calcCrc = crc16_ccitt_kermit(raw, 8);
+	PrintAndLog("Animal ID:     %04u-%012llu", countryCode, NationalCode);
+	PrintAndLog("National Code: %012llu", NationalCode);
+	PrintAndLog("CountryCode:   %04u", countryCode);
+	PrintAndLog("Extended Data: %s", dataBlockBit ? "True" : "False");
+	PrintAndLog("reserved Code: %u", reservedCode);
+	PrintAndLog("Animal Tag:    %s", animalBit ? "True" : "False");
+	PrintAndLog("CRC:           0x%04X - [%04X] - %s", crc16, calcCrc, (calcCrc == crc16) ? "Passed" : "Failed");
+	PrintAndLog("Extended:      0x%X\n", extended);
 	
 	return 1;
 }
@@ -1539,12 +1557,12 @@ int PSKDemod(const char *Cmd, bool verbose)
 		clk=0;
 	}
 	if (invert != 0 && invert != 1) {
-		if (verbose) PrintAndLog("Invalid argument: %s", Cmd);
+		if (g_debugMode || verbose) PrintAndLog("Invalid argument: %s", Cmd);
 		return 0;
 	}
 	uint8_t BitStream[MAX_GRAPH_TRACE_LEN]={0};
 	size_t BitLen = getFromGraphBuf(BitStream);
-	if (BitLen==0) return -1;
+	if (BitLen==0) return 0;
 	uint8_t carrier=countFC(BitStream, BitLen, 0);
 	if (carrier!=2 && carrier!=4 && carrier!=8){
 		//invalid carrier
@@ -1940,7 +1958,7 @@ typedef struct {
 	uint8_t * buffer;
 	uint32_t numbits;
 	uint32_t position;
-}BitstreamOut;
+} BitstreamOut;
 
 bool _headBit( BitstreamOut *stream)
 {
@@ -2270,6 +2288,7 @@ static command_t CommandTable[] =
 	{"buffclear",       CmdBuffClear,       1, "Clear sample buffer and graph window"},
 	{"dec",             CmdDec,             1, "Decimate samples"},
 	{"detectclock",     CmdDetectClockRate, 1, "[modulation] Detect clock rate of wave in GraphBuffer (options: 'a','f','n','p' for ask, fsk, nrz, psk respectively)"},
+	{"fdxbdemod",       CmdFDXBdemodBI    , 1, "Demodulate a FDX-B ISO11784/85 Biphase tag from GraphBuffer"},
 	{"fskawiddemod",    CmdFSKdemodAWID,    1, "Demodulate an AWID FSK tag from GraphBuffer"},
 	//{"fskfcdetect",   CmdFSKfcDetect,     1, "Try to detect the Field Clock of an FSK wave"},
 	{"fskhiddemod",     CmdFSKdemodHID,     1, "Demodulate a HID FSK tag from GraphBuffer"},
@@ -2281,7 +2300,6 @@ static command_t CommandTable[] =
 	{"hexsamples",      CmdHexsamples,      0, "<bytes> [<offset>] -- Dump big buffer as hex bytes"},
 	{"hide",            CmdHide,            1, "Hide graph window"},
 	{"hpf",             CmdHpf,             1, "Remove DC offset from trace"},
-	{"iso11784demod",   CmdIso11784demodBI, 1, "Demodulate a ISO11784/85 Biphase tag from GraphBuffer"},
 	{"load",            CmdLoad,            1, "<filename> -- Load trace (to graph window"},
 	{"ltrim",           CmdLtrim,           1, "<samples> -- Trim samples from left of trace"},
 	{"rtrim",           CmdRtrim,           1, "<location to end trace> -- Trim samples from right of trace"},
@@ -1457,13 +1456,24 @@ int CmdFSKdemodPyramid(const char *Cmd)
 }
 
-// ISO11784/85 demod  (aka animal tag)  BIPHASE, inverted, rf/32,  with preamble of 00000000001 (128bits)
+// FDX-B ISO11784/85 demod  (aka animal tag)  BIPHASE, inverted, rf/32,  with preamble of 00000000001 (128bits)
 // 8 databits + 1 parity (1)
 // CIITT 16 chksum